#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "linux.h"
#include "common.h"


/*------------------------------------------------------------------*/
/*
 * Open a socket.
 * Depending on the protocol present, open the right socket. The socket
 * will allow us to talk to the driver.
 */
int
sockets_open(void)
{
	int ipx_sock = -1;		/* IPX socket			*/
	int ax25_sock = -1;		/* AX.25 socket			*/
	int inet_sock = -1;		/* INET socket			*/
	int ddp_sock = -1;		/* Appletalk DDP socket		*/

	inet_sock=socket(AF_INET, SOCK_DGRAM, 0);
	ipx_sock=socket(AF_IPX, SOCK_DGRAM, 0);
	ax25_sock=socket(AF_AX25, SOCK_DGRAM, 0);
	ddp_sock=socket(AF_APPLETALK, SOCK_DGRAM, 0);
	/*
	 * Now pick any (exisiting) useful socket family for generic queries
	 */
	if(inet_sock!=-1)
		return inet_sock;
	if(ipx_sock!=-1)
		return ipx_sock;
	if(ax25_sock!=-1)
		return ax25_sock;
	/*
	 * If this is -1 we have no known network layers and its time to jump.
	 */
	 
	return ddp_sock;
}

/*********************** WIRELESS SUBROUTINES ************************/

/*------------------------------------------------------------------*/
/*
 * Get the range information out of the driver
 */
int
get_range_info(int		skfd,
	       char *		ifname,
	       iwrange *	range)
{
  struct iwreq		wrq;

  strcpy(wrq.ifr_name, ifname);
  wrq.u.data.pointer = (caddr_t) range;
  wrq.u.data.length = 0;
  wrq.u.data.flags = 0;
  if(ioctl(skfd, SIOCGIWRANGE, &wrq) < 0)
    return(-1);

  return(0);
}



/********************** FREQUENCY SUBROUTINES ***********************/

/*------------------------------------------------------------------*/
/*
 * Convert a floating point the our internal representation of
 * frequencies.
 * The kernel doesn't want to hear about floating point, so we use
 * this custom format instead.
 */
void
float2freq(double	in,
	   iwfreq *	out)
{
  out->e = (short) (floor(log10(in)));
  if(out->e > 8)
    {
      out->m = ((long) (floor(in / pow(10,out->e - 6)))) * 100;
      out->e -= 8;
    }
  else
    {
      out->m = (__u32)in;
      out->e = 0;
    }
}

/*------------------------------------------------------------------*/
/*
 * Convert our internal representation of frequencies to a floating point.
 */
double
freq2float(iwfreq *	in)
{
  return ((double) in->m) * pow(10,in->e);
}


/*********************** ADDRESS SUBROUTINES ************************/

/*
 * This section is mostly a cut & past from net-tools-1.2.0
 * manage address display and input...
 */

/*------------------------------------------------------------------*/
/*
 * Check if interface support the right address types...
 */
int
check_addr_type(int	skfd,
		char *	ifname)
{
  struct ifreq		ifr;

  /* Get the type of interface address */
  strcpy(ifr.ifr_name, ifname);
  if((ioctl(skfd, SIOCGIFADDR, &ifr) < 0) ||
     (ifr.ifr_addr.sa_family !=  AF_INET))
    {
      /* Deep trouble... */
      fprintf(stderr, "Interface %s doesn't support IP addresses\n", ifname);
      return(-1);
    }

#ifdef DEBUG
  printf("Interface : %d - 0x%lX\n", ifr.ifr_addr.sa_family,
	 *((unsigned long *) ifr.ifr_addr.sa_data));
#endif

  /* Get the type of hardware address */
  strcpy(ifr.ifr_name, ifname);
  if((ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) ||
     (ifr.ifr_hwaddr.sa_family != ARPHRD_ETHER))
    {
      /* Deep trouble... */
      fprintf(stderr, "Interface %s doesn't support MAC addresses\n",
	     ifname);
      return(-1);
    }

#ifdef DEBUG
  printf("Hardware : %d - %s\n", ifr.ifr_hwaddr.sa_family,
	 pr_ether(ifr.ifr_hwaddr.sa_data));
#endif

  return(0);
}


/************************* MISC SUBROUTINES **************************/


/*
 * Read /proc/net/wireless to get the latest statistics
 */
int
iw_getstats(char *	ifname,
	    iwstats *	stats)
{
  FILE *	f=fopen("/proc/net/wireless","r");
  char		buf[256];
  char *	bp;
  int		t;
  if(f==NULL)
    return -1;
  /* Loop on all devices */
  while(fgets(buf,255,f))
    {
      bp=buf;
      while(*bp&&isspace(*bp))
	bp++;
      /* Is it the good device ? */
      if(strncmp(bp,ifname,strlen(ifname))==0 && bp[strlen(ifname)]==':')
  	{
	  /* Skip ethX: */
	  bp=strchr(bp,':');
	  bp++;
	  /* -- status -- */
	  bp = strtok(bp, " ");
	  sscanf(bp, "%X", &t);
	  stats->status = (unsigned short) t;
	  /* -- link quality -- */
	  bp = strtok(NULL, " ");
	  if(strchr(bp,'.') != NULL)
	    stats->qual.updated |= 1;
	  sscanf(bp, "%d", &t);
	  stats->qual.qual = (unsigned char) t;
	  /* -- signal level -- */
	  bp = strtok(NULL, " ");
	  if(strchr(bp,'.') != NULL)
	    stats->qual.updated |= 2;
	  sscanf(bp, "%d", &t);
	  stats->qual.level = (unsigned char) t;
	  /* -- noise level -- */
	  bp = strtok(NULL, " ");
	  if(strchr(bp,'.') != NULL)
	    stats->qual.updated += 4;
	  sscanf(bp, "%d", &t);
	  stats->qual.noise = (unsigned char) t;
	  /* -- discarded packets -- */
	  bp = strtok(NULL, " ");
	  sscanf(bp, "%d", &stats->discard.nwid);
	  bp = strtok(NULL, " ");
	  sscanf(bp, "%d", &stats->discard.code);
	  bp = strtok(NULL, " ");
	  sscanf(bp, "%d", &stats->discard.misc);
	  fclose(f);
	  return 0;
  	}
    }
  fclose(f);
  return -1;
}


int freq2channel( float megaFreq, char *channel )
{
  int chFreq[12] = {0,2412,2417,2422,2427,2432,2437,2442,2447,2452,2457,2462};

  for( int i = 1; i < 12; i++ ) {
    if( chFreq[i] == (int)megaFreq ) {
      sprintf( channel, "%d  %d MHz", i, chFreq[i]);
      return 0;
    }
  }
  return -1;
}
    
	
void getMin( int *minSignal, int *minNoise, int *minQual )
{
	*minSignal = 47;
	*minNoise = 47;
	*minQual = 0;
}

char * getToken( char *line, int *i )
{
        static char buf[18];

        if( line[*i] == '\n' )
                return NULL;
        while( line[*i] == ' '){
                (*i)++;
        }
        int j = 0;
        char c = line[*i];
        while( c != ' ' && c != '\n' ) {
                buf[j] = c;
                ++j; ++(*i);
                c = line[*i];
        }
        buf[j] = '\0';
        return buf;
}


int getIps( int count, char **mac, char **ip )
{
	FILE *fp;
	char line[200];
	char hw[20][64]; // store mac address
	char ipAddr[20][64]; // store ip address

	if ( (fp = fopen("/proc/net/arp", "r")) == (FILE *) NULL ) {
                return ( -1 ); 
        }
	int i = 0;
	// read first line which is just the headings
	fgets( line, sizeof(char)*200, fp );
	while( fgets( line, sizeof(char)*200, fp ) != (char *)NULL ) { 
	  int pos = 0;
          char *token;
	  int tokenNumber = 0;
          while( (token = getToken(line, &pos)) != NULL ) {
		if( tokenNumber == 0 )
		  strcpy( ipAddr[i], token );
		if( tokenNumber == 3 )
		  strcpy( hw[i], token );
		++tokenNumber;
	  }
	  ++i;
        }
	int numberOfAddr = i;
	for( int i = 0 ; i < count; i++ ) {
	  for( int j = 0; j < numberOfAddr; j++ ) {
		if( strcmp(mac[i], hw[j] ) == 0 ) {
		  strcpy( ip[i], ipAddr[j]);
		  break;
		}
	  }
	}
	  
	return 0;
}
