/**************************************************************************
 The GUI for displaying the receiving signal, noise and quality levels of
 a 802.11b network card.
 Programmer: Ginny Mak, makg@cs.pdx.edu
 Location: Portland, Oregon.
 Date: October 7, 2001.
**************************************************************************/ 

/*
 * Copyright (c) 2001, 2002 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include<stdio.h>
#include<stdlib.h>
#include<string>
#include<unistd.h>
#include<FL/Fl.h>
#include<Fl/Fl_Window.h>
#include<FL/Fl_Box.h>
#include<FL/Enumerations.h>
#include<FL/fl_draw.h>
#include<FL/Fl_Value_Slider.h>
#include<FL/Fl_Output.h>
#include<FL/Fl_Button.h>
#include<FL/Fl_Menu.h>
#include<FL/Fl_Menu_Bar.h>
#include<FL/Fl_Menu_Item.h>
#include<FL/Fl_Counter.h>
#include<FL/Fl_Scroll.h>
#include<FL/Fl_Group.h>
#include<errno.h>
#include "cSigStat.h"
#include "wscantypes.h"
#include "common.h"

#define FAILURE -1
#define SUCCESS 0
#define MAX_HISTORY 1000 // store 1000 history points before starting all over.
#define MAX_DISPLAY 24
#define MAXIFS 40

//------------------------------------------------------------

/**********************************
 * VARIABLES DECLARATIONS         *
 **********************************/
bool change = false;       // signal to go callback function that user initiates a change
bool noRange[MAX_DISPLAY];
char *iflist[MAXIFS];

cSigStat *sigStat;

operationInfo opInfo;
sigtuple signalTuple;
Fl_Menu_Item *ifMenuItems;

char ifname[128];
double pollInterval = 2.0;  // default value of polling interval
int prism2 = 0;
int apscan = 0;	    /* even if in ad hoc mode, force managed mode */

// each carries MAC/essid address
/*
char listItem0[100];
char listItem1[100];
char listItem2[100];
char listItem3[100];
char listItem4[100];
char listItem5[100];
char *ipList[] = {listItem0, listItem1, listItem2, listItem3, listItem4, listItem5};
*/
char *ipList[MAX_DISPLAY];

int listIndex = 0; // index the addresses in ipList

int numberOfDisplay = 0;

sigtuple sigTupleAr[MAX_DISPLAY]; // for getting a table of statistics

#define MAXMAC 64 
#define MAXIP  20
#define MAXESSID 33
char macString[MAX_DISPLAY][MAXMAC];
char ipString[MAX_DISPLAY][MAXIP];
char essidString[MAX_DISPLAY][MAXIP];
char *wepString[MAX_DISPLAY];
char channelString[MAX_DISPLAY][32];
int level[MAX_DISPLAY];
int noise[MAX_DISPLAY];
int quality[MAX_DISPLAY];

bool managed = false;
bool adhoc = false;
int currentMode = 0;
int mode;

// These are for drawing the history points on the meter window
int resetFlag = 0;
int myIndex = 0;
int ssHistory[MAX_HISTORY];
int snHistory[MAX_HISTORY];
int sqHistory[MAX_HISTORY];
int ssY, snY, sqY; // y coordinates for the history points
int yRange = 100;  // the range of progress bar in pixels
int yEnd = 160;    // end point of progress bar

#define LOGPATHNAME WSCANLOG

char wscanhack[1024];

//---------------------------------------------------------------------

/***********************************
 * FUNCTIONS DECLARATION           *
 ***********************************/
void getOpInfo( cSigStat *sigStat, operationInfo * info );
void getStat( cSigStat *sigStat);
void startWindow();
void updateDisplay(void*);
Fl_Color genColor(int level);
void popList();
void message(char * msg);
void ask(char *msg);
int startInitialize( char *ifname );
int start( char *ifname );


//------------------------------------------------------------------

/******************************************************
 polling interval window declaration and definitions
******************************************************/
void pollInterval_cb(Fl_Widget *w, void  *)
{
  Fl_Counter * cw = (Fl_Counter *)w;
  pollInterval = cw->value();
}
  
 
class counterWindow:public Fl_Window {
public:
  counterWindow(int w, int h, const char *label=0 );
  ~counterWindow();
private:
  void initialize();
  Fl_Counter *pollIntervalCounter;
};

counterWindow::counterWindow(int w, int h, const char *label=0):Fl_Window(w,h,label)
{
  initialize();
}


counterWindow::~counterWindow()
{
  delete pollIntervalCounter;
}

void counterWindow::initialize()
{
  this->set_modal();
  this->color(255);
  pollIntervalCounter = new Fl_Counter(20, 20, 150,20,"Polling Interval (sec)");
  pollIntervalCounter->type(FL_SIMPLE_COUNTER);
  pollIntervalCounter->value(pollInterval);
  pollIntervalCounter->minimum(1.0);
  pollIntervalCounter->maximum(100.0);
  pollIntervalCounter->step(1.0);
  pollIntervalCounter->color(FL_YELLOW);
  pollIntervalCounter->labelcolor(FL_BLACK);
  pollIntervalCounter->selection_color(255);
  pollIntervalCounter->labelfont(FL_HELVETICA_BOLD);
  pollIntervalCounter->callback(pollInterval_cb);
}

counterWindow *myCounterWindow;

//--------------------------------------------------------------

/************************************************
 openWindow declaration 
************************************************/
// Menu items in the menu bar on the opening window

Fl_Menu_Item openMenuItems[] = {
  {"List"},
  {"Poll Interval"},
  {"Detail"},
  {"Close"},
  {0}
};

class openWindow:public Fl_Window {
public:
  openWindow(int w, int h, int numOfDisplay, const char*label=0);
  ~openWindow();
  double getPollingInterval() const;
  void draw();
  void setQuality(int myIndex, double aValue);
  void setNoRangeQuality( int myIndex, int aValue );
  void setAddr( int myIndex );

private:
  void initialize();
  int listLen;
  char intString[10];
  Fl_Menu_Bar *menuBar;
  Fl_Box *label;
  Fl_Scroll *scroll;
  Fl_Output *dummy[MAX_DISPLAY];
  Fl_Value_Slider *qualityAr[MAX_DISPLAY];
  Fl_Output *openWindowAddr[MAX_DISPLAY];
  Fl_Output *noRangeQuality[MAX_DISPLAY];
  
};

openWindow *window;


/**********************************************************************
 meterWindow declarations 
**********************************************************************/
// menu items in the menu bar on the meter window
Fl_Menu_Item menuItems[] = {
  {"Change"},
  {"Reset"},
  {"Close"},
  {0}
};

//-------------------------------------------------------

/******************************************
 Class meterWindow
*******************************************/
class meterWindow:public Fl_Window
{
  static const bool maxDisplay = 5;
public:
  meterWindow(int w, int h, char *label=0 );
  ~meterWindow();
  void draw();
  void setSSvalue(double aValue);
  void setSNvalue(double aValue);
  void setSQvalue(double aValue);
  void setMode(char *aMode);
  void setEssid(char *aEssid);
  void setChannel(char *aChannel);
  void setMac(char *aMac);
  void setIp(char *aIp);
  void setNoRangeLevel(int aLevel);
  void setNoRangeNoise(int aNoise);
  void setNoRangeQuality(int aQuality);
  void reset();

private:
  void initialize();
  char intString[10];
  int ssX, snX, sqX; // x coordinates of the history points for level, noise and quality
  double min;
  double max;
  double aStep;
  double aValue;
  int labelSize;
  Fl_Menu_Bar *menubar;
  Fl_Value_Slider *ssMeter;
  Fl_Value_Slider *snMeter;
  Fl_Value_Slider *sqMeter;
  Fl_Output *noRangeLevel;
  Fl_Output *noRangeNoise;
  Fl_Output *noRangeQuality;
  Fl_Output *mode;
  Fl_Output *essid;
  Fl_Output *channel;
  Fl_Output *mac;
  Fl_Output *ip;
};

meterWindow *myMeterWindow;

//-----------------------------------------------------------------------

/***************************************************
 * CLASS DEFINITIONS AND CALLBACK FUNCTIONS        *
 ***************************************************/

/*************************************************************************
 openWindow definition and callback functions
*************************************************************************/
//--------------------|
// CALLBACK FUNCTIONS |
//--------------------|
void window_cb(Fl_Widget*, void*) {
  ask( "Want to quit?" );
}          

void openMenu_cb(Fl_Widget *w, void *data)
{
  Fl_Menu_* mw = (Fl_Menu_*)w;
  const Fl_Menu_Item* m = mw->mvalue();
  if( strcmp( m->label(), "Close")==0 ) {
	ask( "Want to quit?" );
  }
  else if( strcmp( m->label(), "Poll Interval" )==0 ){
	myCounterWindow->show();
  }
  else if( strcmp( m->label(), "List" ) == 0 ) {
	popList();
  }
  else if( strcmp( m->label(), "Detail")==0 ) {
	pollInterval = window->getPollingInterval();
	getStat( sigStat );
	updateDisplay(0);
	
	window->hide();
	myMeterWindow->show();

  }
}


//-------------------------------|
// CLASS openWindow DEFINITION   |
//-------------------------------|
openWindow::openWindow(int w, int h, int numOfDisplay,const char *label=0 ):Fl_Window(w,h,label)
{
  listLen = numOfDisplay;
  initialize();
}

openWindow::~openWindow()
{
  delete menuBar;
  delete label;
  for( int i = 0; i < MAX_DISPLAY; i++ ) {
	delete qualityAr[i];
	delete openWindowAddr[i];
	delete noRangeQuality[i];
	delete dummy[i];
  }
  sigStat->wifi_close();
}

void openWindow::initialize()      
{
  double min = 0;
  double max = 100.0;
  double aStep = 0;
  int labelSize = 10;

  this->callback(window_cb);
 
  menuBar = new Fl_Menu_Bar(0,0,240,20,0);
  menuBar->menu(openMenuItems);
  menuBar->textfont(FL_HELVETICA_BOLD);
#ifdef IPAQ
  menuBar->textsize(8);
#else
  menuBar->textsize(10);
#endif
  menuBar->textcolor(FL_BLACK);
  menuBar->color(255);
  menuBar->callback(openMenu_cb);

  // heading
  Fl_Box *label = new Fl_Box(5,22, 50, 7, "QUALITY");
  label->labelcolor(255);
#ifdef IPAQ
  label->labelsize(8);
#else
  label->labelsize(12);
#endif
  
  scroll = new Fl_Scroll( 0, 55, 240, 250 );
  scroll->scrollbar.align( Fl_Scroll::VERTICAL_ALWAYS );
  scroll->box( FL_NO_BOX );
  scroll->align( FL_ALIGN_RIGHT );
  scroll->color( FL_BLACK );

  // layout the maximum number of quality meters
  int startY = 55;
#ifdef IPAQ
  int stepY = 44;
#else
  int stepY = 50;
#endif
  for(int i = 0; i < MAX_DISPLAY; i++ ) {
	qualityAr[i] = new Fl_Value_Slider( 0, startY+i*stepY, 225, 15, 0);
	qualityAr[i]->type(FL_HOR_FILL_SLIDER);
	qualityAr[i]->color(255); // white
	qualityAr[i]->labelsize(labelSize);
	qualityAr[i]->labelcolor(255);
	qualityAr[i]->range(max, min);
	qualityAr[i]->step(aStep);
	qualityAr[i]->minimum(min);
	qualityAr[i]->maximum(max);
	dummy[i] = new Fl_Output( 225, startY+i*stepY, 75, 15, 0 );
	dummy[i]->color( FL_BLACK );
	if( !noRange[i] ) {
	  qualityAr[i]->value(quality[i]);
	  qualityAr[i]->selection_color(genColor(quality[i]));
	}
	
#ifdef IPAQ
	openWindowAddr[i] = new Fl_Output( 0, startY+i*stepY+15, 300, 15,0 );
	openWindowAddr[i]->textsize(6);
#else
	openWindowAddr[i] = new Fl_Output( 0, startY+i*stepY+15, 300, 20,0 );
	openWindowAddr[i]->textsize(10);
#endif
	openWindowAddr[i]->color(150);
	openWindowAddr[i]->textcolor(FL_BLACK);
	openWindowAddr[i]->value(ipList[i]);
	
	scroll->add( openWindowAddr[i] );
	scroll->add( dummy[i] );
	
#ifdef IPAQ
	noRangeQuality[i] = new Fl_Output( 0, startY+i*stepY+30, 300, 14, 0 );
	noRangeQuality[i]->textsize(8);
#else
	noRangeQuality[i] = new Fl_Output( 0, startY+i*stepY+35, 300, 14, 0 );
	noRangeQuality[i]->textsize(12);
#endif

	noRangeQuality[i]->color(FL_YELLOW);
	noRangeQuality[i]->textcolor(FL_BLACK);
	
	if( managed == true ) {
	  sprintf( intString, "%d", signalTuple.quality );
	  noRangeQuality[0]->value( intString );
	}
	else {
	  sprintf( intString, "%d", sigTupleAr[i].quality );
	  noRangeQuality[i]->value( intString );
	}
	scroll->add( noRangeQuality[i] );
  }
}

inline double openWindow::getPollingInterval() const
{
  return pollInterval;
}


void openWindow::draw()
{
  // draw the color coded graduation 
  // and graduation labels at the bottom of the window
  Fl_Window::draw();
  fl_color(FL_RED);
#ifdef IPAQ
  fl_rectf(50,35,48,4);
#else
  fl_rectf(50,35,48,4);
#endif
  fl_color(FL_BLUE);
#ifdef IPAQ
  fl_rectf(98,35,48,4);
#else
  fl_rectf(98,35,48,4);
#endif
  fl_color(FL_YELLOW);
#ifdef IPAQ
  fl_rectf(146,35,48,4);
#else
  fl_rectf(146,35,48,4);
#endif 
  fl_color(FL_GREEN);
#ifdef IPAQ
  fl_rectf(194,35,48,4);
#else
  fl_rectf(194,35,48,4);
#endif
  fl_color(255);
  fl_font(FL_HELVETICA,10);
#ifdef IPAQ
  fl_draw("0",50,50);
  fl_draw("25", 93, 50);
  fl_draw("50", 140, 50);
  fl_draw("75", 184, 50);
  fl_draw("100", 220,50);
#else
  fl_draw("0",50,50);
  fl_draw("25", 93, 50);
  fl_draw("50", 140, 50);
  fl_draw("75", 184, 50);
  fl_draw("100", 220,50);
#endif
  
}


// Set the quality of a specific quality meter on the open window
inline void openWindow::setQuality( int myIndex, double aValue )
{
  qualityAr[myIndex]->value(aValue);
  qualityAr[myIndex]->selection_color( genColor((int)aValue) );
}


inline void openWindow::setNoRangeQuality( int myIndex, int aValue )
{
  if( aValue < 0 )
	noRangeQuality[myIndex]->value("");
  else {
	sprintf( intString, "%d", aValue );
	noRangeQuality[myIndex]->value(intString);
  }
}

inline void openWindow::setAddr( int myIndex )
{
  openWindowAddr[myIndex]->value( ipList[myIndex] );
}

//---------------------------------------------------------------------------

/***************************************************************
  meterWindow definition and callback functions
***************************************************************/
//---------------------------------------------|
// CALLBACK FUNCTIONS FOR THE METER WINDOW     |
//---------------------------------------------|

void menu_cb(Fl_Widget *w, void *data)
{
  Fl_Menu_* mw = (Fl_Menu_*)w;
  const Fl_Menu_Item* m = mw->mvalue();
  if( strcmp( m->label(), "Close")==0 ) {
	ask( "Want to quit?" );
  }
  if( strcmp( m->label(), "Change") == 0 ) {
	change = true;
	myMeterWindow->hide();
	window->show();
  }
  else if( strcmp( m->label(), "Reset" ) == 0 ) {
	myMeterWindow->setSSvalue(0.0);
	myMeterWindow->setSNvalue(0.0);
	myMeterWindow->setSQvalue(0.0);
	myMeterWindow->setNoRangeLevel(-1);
	myMeterWindow->setNoRangeNoise(-1);
	myMeterWindow->setNoRangeQuality(-1);
	resetFlag = 1;
	myIndex = 0;
	myMeterWindow->redraw();
  }
}

/*
 * log button pressed - write vars to and exec
 * shellscript
 */
void log_cb( Fl_Widget *w, void *data )
{
  char wscanhack[1024];
  char *ifn;
  char *mac;
  char *essid;
  char *chan;
  int  i = listIndex;
  int sig_level, sig_noise, sig_quality;

  essid = opInfo.essid;
  chan = opInfo.channel;
  if( managed == true ) {
        ifn = ifname;
        mac = opInfo.mac;
	sig_level = level[0];
	sig_noise = noise[0];
	sig_quality = quality[0];
  }
  else {
        if (i > MAX_DISPLAY)
          i = 0;
        ifn = ipString[i];
        mac = macString[i];
	sig_level = level[i];
	sig_noise = noise[i];
	sig_quality = quality[i];
	if (apscan) {
		essid = essidString[i];
		chan = channelString[i];
		ifn = wepString[i];
	}
  }
  sprintf( wscanhack, "\"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" %d %d %d\n", LOGPATHNAME        ,opInfo.mode, essid, mac, chan, ifn, 
	sig_level, sig_noise, sig_quality);

  (void) system( wscanhack );
}

void close_cb(Fl_Widget *w, void *data)
{
  ask( "Want to quit" );
}
  
meterWindow::meterWindow(int w, int h, char *label=0):Fl_Window(w,h,label)
{
  initialize();
}

meterWindow::~meterWindow()
{
  delete menubar;
  delete ssMeter;
  delete snMeter;
  delete sqMeter;
  delete mode;
  delete essid;
  delete channel;
  delete mac;
  delete ip;
  delete noRangeLevel;
  delete noRangeNoise;
  delete noRangeQuality;
}

void meterWindow::initialize()
{
  this->color(FL_BLACK);
  this->callback(close_cb);
  
  min = 100.0;
  max = 0;
  aStep = 0;  // default
  labelSize = 12; // font size of label
  ssX = 90; snX = 140; sqX = 190;

  menubar = new Fl_Menu_Bar(0,0,240,20, 0);
  menubar->menu(menuItems);
  menubar->color(255);
  menubar->textfont(FL_HELVETICA_BOLD);
  menubar->callback(menu_cb);

#ifdef IPAQ
  Fl_Button *log = new Fl_Button(197 , 30, 35, 20, "log");
#else
  Fl_Button *log = new Fl_Button(197 , 30, 38, 20, "log");
#endif
  
  log->type(0);
  log->color(FL_RED);
  log->labelcolor(255);
  log->labelsize(10);
  log->callback(log_cb);

  ssMeter = new Fl_Value_Slider(60, 30, 27, 130, "signal");
  ssMeter->type(FL_VERT_FILL_SLIDER);
  ssMeter->color(255); // white
  ssMeter->labelsize(labelSize);
  ssMeter->range(max, min);
  ssMeter->step(aStep);
  ssMeter->minimum(min);
  ssMeter->maximum(max);
  if( !noRange[0] )
    ssMeter->value(level[0]);

  snMeter = new Fl_Value_Slider(110, 30, 27, 130, "noise");
  snMeter->type(FL_VERT_FILL_SLIDER);
  snMeter->color(255);
  snMeter->labelsize(labelSize);
  snMeter->range(max, min);
  snMeter->step(aStep);
  snMeter->minimum(min);
  snMeter->maximum(max);
  if( !noRange[0] )
    snMeter->value(noise[0]);

  sqMeter = new Fl_Value_Slider(160, 30, 27, 130, "quality");
  sqMeter->type(FL_VERT_FILL_SLIDER);
  sqMeter->color(255);
  sqMeter->labelsize(labelSize);
  sqMeter->range(max,min);
  sqMeter->step(aStep);
  sqMeter->minimum(min);
  sqMeter->maximum(max);
  if( !noRange[0] )
    sqMeter->value(quality[0]);

  noRangeLevel = new Fl_Output(53, 165, 40, 15,0);
  noRangeLevel->color(FL_YELLOW);
  noRangeLevel->textcolor(FL_BLACK);
  noRangeLevel->textsize(10);

  noRangeNoise = new Fl_Output(103, 165, 40, 15,0);
  noRangeNoise->color(FL_YELLOW);
  noRangeNoise->textcolor(FL_BLACK);
  noRangeNoise->textsize(10);

  noRangeQuality = new Fl_Output(153, 165, 40, 15,0);
  noRangeQuality->color(FL_YELLOW);
  noRangeQuality->textcolor(FL_BLACK);
  noRangeQuality->textsize(10);

  if( managed == true ) {
    sprintf( intString, "%d", signalTuple.signal );
    noRangeLevel->value( intString );
    sprintf( intString, "%d", signalTuple.noise );
    noRangeNoise->value( intString );
    sprintf( intString, "%d", signalTuple.quality );
    noRangeQuality->value( intString );
  }
  else { // ad-hoc
    sprintf( intString, "%d", sigTupleAr[0].signal );
    noRangeLevel->value( intString );
    sprintf( intString, "%d", sigTupleAr[0].noise );
    noRangeNoise->value( intString );
    sprintf( intString, "%d", sigTupleAr[0].quality );
    noRangeQuality->value( intString );
  }

  // graduation labels of the meters
  Fl_Box *box0 = new Fl_Box(40, 155, 5,5,"0");
  box0->labelcolor(255);
  box0->labelsize(10);
  Fl_Box *box25 = new Fl_Box(40, 130, 5,5,"25");
  box25->labelcolor(255);
  box25->labelsize(10);
  Fl_Box *box50 = new Fl_Box(40, 105, 5,5,"50");
  box50->labelcolor(255);
  box50->labelsize(10);
  Fl_Box *box75 = new Fl_Box(40, 80, 5,5,"75");
  box75->labelcolor(255);
  box75->labelsize(10);
  Fl_Box *box100 = new Fl_Box(40, 55, 5,5,"100");
  box100->labelcolor(255);
  box100->labelsize(10);
  

  Fl_Box *labelSignal = new Fl_Box(58, 190, 25, 5, "SIGNAL");
  labelSignal->labelcolor(255);
#ifdef IPAQ
  labelSignal->labelsize(8);
#else
  labelSignal->labelsize(10);
#endif
  Fl_Box *labelNoise = new Fl_Box(108, 190, 25, 5, "NOISE");
  labelNoise->labelcolor(255);
#ifdef IPAQ
  labelNoise->labelsize(8);
#else
  labelNoise->labelsize(10);
#endif
  Fl_Box *labelQuality = new Fl_Box(160, 190, 25, 5, "QUALITY");
  labelQuality->labelcolor(255);
#ifdef IPAQ
  labelQuality->labelsize(8);
#else
  labelQuality->labelsize(10);
#endif

 
  Fl_Box *modeLabel = new Fl_Box(10, 205, 15, 15, "Mode");
  modeLabel->labelcolor(255);
#ifdef IPAQ
  modeLabel->labelsize(8);
#else
  modeLabel->labelsize(10);
#endif
  mode = new Fl_Output(40, 205, 193, 15,0);
  mode->color(255);
  mode->textcolor(FL_BLACK);
#ifdef IPAQ
  mode->textsize(8);
#else
  mode->textsize(10);
#endif
  mode->value(opInfo.mode);

#ifdef IPAQ
  Fl_Box *essidLabel = new Fl_Box(10, 220, 15, 15, "ESSID");
#else
  Fl_Box *essidLabel = new Fl_Box(10, 225, 15, 15, "ESSID");
#endif
  essidLabel->labelcolor(255);
#ifdef IPAQ
  essidLabel->labelsize(8);
  essid = new Fl_Output(40, 220, 193, 15, 0);
  essid->textsize(8);
#else
  essidLabel->labelsize(10);
  essid = new Fl_Output(40, 225, 193, 15, 0);
  essid->textsize(10);
#endif
  essid->color(255);
  essid->textcolor(FL_BLACK);
  essid->value(opInfo.essid);

#ifdef IPAQ
  Fl_Box *channelLabel = new Fl_Box(10, 235, 15, 15, "Channel");
#else
  Fl_Box *channelLabel = new Fl_Box(10, 245, 15, 15, "Channel");
#endif
  channelLabel->labelcolor(255);
#ifdef IPAQ
  channelLabel->labelsize(6);
  channel = new Fl_Output(40, 235, 193, 15, 0);
  channel->textsize(8);
#else
  channelLabel->labelsize(8);
  channel = new Fl_Output(40, 245, 193, 15, 0);
  channel->textsize(10);
#endif
  channel->color(255);
  channel->textcolor(FL_BLACK);
  channel->value(opInfo.channel);

  // The default display in the meter window is the first one
  // of the addresses
#ifdef IPAQ
  Fl_Box *macLabel = new Fl_Box(10, 250, 15, 15, "MAC");
#else
  Fl_Box *macLabel = new Fl_Box(10, 265, 15, 15, "MAC");
#endif
  macLabel->labelcolor(255);
#ifdef IPAQ
  macLabel->labelsize(8);
  mac = new Fl_Output(40, 250, 193, 15, 0);
  mac->textsize(8);
#else
  macLabel->labelsize(10);
  mac = new Fl_Output(40, 265, 193, 15, 0);
  mac->textsize(10);
#endif
  mac->color(255);
  mac->textcolor(FL_BLACK);
  mac->value(macString[0]); 

  // The default display in the meter window is the first one
  // of the addresses
#ifdef IPAQ
  Fl_Box *ipLabel = new Fl_Box(15, 265, 10, 15, "misc");
#else
  Fl_Box *ipLabel = new Fl_Box(10, 285, 15, 15, "misc");
#endif
  ipLabel->labelcolor(255);
#ifdef IPAQ
  ipLabel->labelsize(8);
  ip = new Fl_Output(40, 265, 193, 15, 0);
  ip->textsize(8);
#else
  ipLabel->labelsize(10);
  ip = new Fl_Output(40, 285, 193, 15, 0);
  ip->textsize(10);
#endif
  ip->color(255);
  ip->textcolor(FL_BLACK);
  
  if( managed == true )  {
    ip->value( ifname );
  }
  else {
    ip->value(ipString[0]);
  }
}

void meterWindow::draw()
{
  Fl_Window::draw();
  fl_color(FL_RED);
#ifdef IPAQ
  fl_rectf(25, 135, 2, 25);
#else
  fl_rectf(30, 135, 2, 25);
#endif
  fl_color(FL_BLUE);
#ifdef IPAQ
  fl_rectf(25, 110, 2, 25);
#else
  fl_rectf(30, 110, 2, 25);
#endif
  fl_color(FL_YELLOW);
#ifdef IPAQ
  fl_rectf(25, 85, 2, 25);
#else
  fl_rectf(30, 85, 2, 25);
#endif
  fl_color(FL_GREEN);
#ifdef IPAQ
  fl_rectf(25, 60, 2, 25);
#else
  fl_rectf(30, 60, 2, 25);
#endif

  // Draw history points
  if(myIndex != 0 && !resetFlag) {
    fl_color(255);
    for(int i = 0; i < myIndex; i++) {
      fl_point(ssX, ssHistory[i]);
      fl_point(snX, snHistory[i]);
      fl_point(sqX, sqHistory[i]);
    }
  }
}

inline void meterWindow::setSSvalue(double aValue)
{
  ssMeter->value( aValue );
  ssMeter->selection_color(genColor((int)aValue));
}

inline void meterWindow::setSNvalue(double aValue)
{
  snMeter->value( aValue );
  snMeter->selection_color(genColor((int)aValue));
}

inline void meterWindow::setSQvalue(double aValue)
{
  sqMeter->value( aValue );
  sqMeter->selection_color(genColor((int)aValue));
}


inline void meterWindow::setMode(char *aMode)
{
  mode->value(aMode);
}


inline void meterWindow::setEssid(char *aEssid)
{
  essid->value(aEssid);
}


inline void meterWindow::setChannel(char *aChannel)
{
  channel->value(aChannel);
}


inline void meterWindow::setMac(char *aMac)
{
  mac->value(aMac);
}

inline void meterWindow::setIp(char *aIp)
{
  ip->value(aIp);
}
  
inline void meterWindow::setNoRangeLevel( int aLevel )
{
  if( aLevel < 0 )
	noRangeLevel->value(" ");
  else {
	sprintf( intString, "%d", aLevel );
	noRangeLevel->value( intString);
  }
}

inline void meterWindow::setNoRangeNoise( int aNoise )
{
  if( aNoise < 0 )
	noRangeNoise->value(" ");
  else {
	sprintf( intString, "%d", aNoise );
	noRangeNoise->value( intString);
  }
}

inline void meterWindow::setNoRangeQuality( int aQuality )
{
  if( aQuality < 0 )
	noRangeQuality->value(" ");
  else {
	sprintf( intString, "%d", aQuality );
	noRangeQuality->value( intString);
  }
}


void meterWindow::reset()
{
  if(!resetFlag) {
	setSSvalue(0.0);
	setSNvalue(0.0);
	setSQvalue(0.0);
	setNoRangeLevel(-1);
	setNoRangeNoise(-1);
	setNoRangeQuality(-1);
	resetFlag = 1;
	redraw();
  }
}

//=============================================================================

/******************************************************************************
 SOME UTILITY FUNCTIONS
******************************************************************************/

int num2modeString( int mode, char *modeString )
{
  if( mode != MANAGED_MODE && mode != ADHOC_MODE ) 
	return FAILURE;
  if( mode == MANAGED_MODE )
	strcpy(modeString, "managed");
  else  {
	if (apscan)
	  strcpy(modeString, "apscan");
	else
	  strcpy(modeString, "ad-hoc");
  }
  return SUCCESS;
}

char *chan2String (int c)
{
	static char chanbuf[1024];
	int chanbase = 2412;

        chanbase = chanbase + ((c - 1) * 5);

        sprintf(chanbuf, "%d:_%d_Mhz", c, chanbase);
        return(chanbuf);
}

//----------------------------------------------------------

void getOpInfo(cSigStat *sigStat, operationInfo *info)
{
  int mode = -1;
  char *mp;

  adhoc = false;
  managed = false;
  if((mode=sigStat->wifi_getmode()) == FAILURE) {
	message("Unsupported operating mode.\n");
  	sigStat->wifi_close();  /* let's close the interface, what say */
	exit(FAILURE);
  }

  if( mode == 1 )
      adhoc = true;
  else
      managed = true;

  num2modeString( mode, info->mode );

  info->essid = sigStat->wifi_getessid();

  if( managed == true ) {
    unsigned char mac[6];
    sigStat->wifi_getmac( mac );
    mp = getmacString(mac, MAXMAC);
    strcpy(info->mac,  mp);
  }

  if( (currentMode == 0) || (currentMode == 1 && managed) 
	|| (currentMode == 2 && adhoc) ) {
    strcpy( info->channel, sigStat->wifi_getchannel() );
  }
  if( currentMode == 0 ) {
    sigStat->wifi_getrange( &(info->maxSignal),
			  &(info->maxNoise),
			  &(info->maxQuality), 
			  &(info->minSignal), 
			  &(info->minNoise), 
			  &(info->minQuality) 
			);

    if( info->maxSignal == 0 )
	  noRange[0] = true;
    else
	  noRange[0] = false;
  }
}


//------------------------------------------------------------------

void getStat( cSigStat *sigStat )
{

  getOpInfo( sigStat,&opInfo);

  // managed mode
  if( managed == true ) {
        numberOfDisplay = 1;
	sigStat->wifi_getsignal( &(signalTuple.signal),
					 &(signalTuple.noise),
					 &(signalTuple.quality) );

	/* TBD. make minmax check in wifi_getsignal and wifi_gettable
	*/
        if( signalTuple.signal < opInfo.minSignal )
          signalTuple.signal = opInfo.minSignal;
        if( signalTuple.noise < opInfo.minNoise )
          signalTuple.noise = opInfo.minNoise;
        if( signalTuple.quality < opInfo.minQuality )
          signalTuple.quality = opInfo.minQuality;

        if( noRange[0] ) {
          level[0] = 0;
          noise[0] = 0;
          quality[0] = 0;
        }
        else {  
          level[0] = percent( signalTuple.signal, opInfo.minSignal,
                                                        opInfo.maxSignal );
          noise[0] = percent( signalTuple.noise, opInfo.minNoise,
                                                        opInfo.maxNoise );
          quality[0] = percent( signalTuple.quality,
                                                        opInfo.minQuality,
                                                        opInfo.maxQuality );
        }

	sigStat->wifi_getmac( signalTuple.mac );
	strcpy( macString[0], getmacString( signalTuple.mac, MAXMAC));

	/* BUG.  make sure these strings cannot overflow ipList buffer
	 */
	if( strlen( macString[0]) + 2 + strlen( opInfo.essid ) < 100 ) {
	  strcpy( ipList[0], macString[0] );
	  strncat( ipList[0], "  ", 2);
	  strncat( ipList[0], opInfo.essid, strlen(opInfo.essid) );
	}
  }

  // ad hoc mode
  else {
	numberOfDisplay = sigStat->wifi_gettable( MAX_DISPLAY, sigTupleAr );

	for (int i = 0; i < numberOfDisplay; i++ ) {

	  strcpy(macString[i], getmacString( sigTupleAr[i].mac, MAXMAC));
          if (apscan) {
		char *s;

	  	strcpy(essidString[i], sigTupleAr[i].essid);
		s = chan2String(sigTupleAr[i].channel);
	  	strncpy(channelString[i], s, 32);
                if (sigTupleAr[i].wep) {
			wepString[i] = "wep";
		}
		else {
			wepString[i] = "nowep";
		}
	  }
	  else
  	  	strcpy(ipString[i], getip(sigTupleAr[i].ip));
	  /* BUG. check this and make sure it doesn't overflow
	   * the string space.
	  */
	  if( strlen( macString[i]) + 1 + strlen(ipString[i]) + strlen(opInfo.essid) < 100 ) {
	    strcpy(ipList[i], macString[i] );
	    strncat(ipList[i], " ", 1 );
            if (apscan) {
	      strncat(ipList[i], " ", 1 );
	      strncat(ipList[i], essidString[i], strlen(essidString[i]) );
	      strncat(ipList[i], " ", 1 );
	      strncat(ipList[i], channelString[i], strlen(channelString[i]) );
	    }
	    else {
	      strncat(ipList[i], ipString[i], strlen(ipString[i]));
	      strncat(ipList[i], " ", 1 );
	      strncat(ipList[i], opInfo.essid, strlen(opInfo.essid) );
            }
	  }

	  if( opInfo.maxSignal == 0 ) { // no range
	        noRange[i] = true;
		level[i] = sigTupleAr[i].signal;
		noise[i] = sigTupleAr[i].noise;
		quality[i] = sigTupleAr[i].quality;
	  }
	  else {
		noRange[i] = false;
		level[i] = percent( sigTupleAr[i].signal, 
						  opInfo.minSignal,
						  opInfo.maxSignal );
		noise[i] = percent( sigTupleAr[i].noise, 
						  opInfo.minNoise,
						  opInfo.maxNoise );
		quality[i] = percent( sigTupleAr[i].quality, 
							opInfo.minQuality,
							opInfo.maxQuality );
	  } 
	}
      }
}
		  
		
//-----------------------------------------------------------

void updateDisplay(void *)
{
  int ssY, snY, sqY;

  getStat(sigStat);
  resetFlag = 0;
  if( myIndex == MAX_HISTORY )
	myIndex = 0;
  
  // record into history only if noRange[listMyIndex] is 0
  if( !noRange[listIndex] ) {
	ssY = (int)(yEnd - ((double)yRange/100)*level[listIndex]);
	ssHistory[myIndex] = ssY;
	snY = (int)(yEnd - ((double)yRange/100)*noise[listIndex]);
	snHistory[myIndex] = snY;
	sqY = (int)(yEnd - ((double)yRange/100)*quality[listIndex]);
	sqHistory[myIndex] = sqY;
	myIndex++;
  }
  
  // update the quality meters on the open window
  for(int i = 0; i < numberOfDisplay; i++ ) {
	if( !noRange[i] ) {
	  window->setQuality( i, quality[i] );
	}
	else {
	  window->setQuality( i, 0 );
	}
        if( managed == true )  
	  window->setNoRangeQuality( i, signalTuple.quality );
	else 
	  window->setNoRangeQuality( i, sigTupleAr[i].quality );

	window->setAddr( i );
  }
  for( int i = numberOfDisplay; i < MAX_DISPLAY; i++ ) {
	window->setQuality( i, 0 );
	window->setNoRangeQuality( i, -1 );
	strcpy( ipList[i], "" );
	window->setAddr( i );
  }

  // update all the meters on the meter window
  // It will be the meters for a specific address

  // set the mode and the channel
  //
  if( (currentMode == 0 ) || (currentMode == 1 && managed) || (currentMode == 2 && adhoc ) ) {
    myMeterWindow->setMode( opInfo.mode );
    if (apscan)
      myMeterWindow->setChannel( channelString[listIndex]);
    else
      myMeterWindow->setChannel( opInfo.channel );
  }

  // set the essid
  if (apscan) {
    myMeterWindow->setEssid( essidString[listIndex] );
  }
  else {
    myMeterWindow->setEssid( opInfo.essid );
  }

  myMeterWindow->setMac( macString[listIndex] );
  
  if( managed == true )  
    myMeterWindow->setIp( ifname );
  else {
    if (apscan) {
      myMeterWindow->setIp( wepString[listIndex] );
    }
    else {
      myMeterWindow->setIp( ipString[listIndex] );
    }
  }

  if( !noRange[listIndex] ) { // has range
	myMeterWindow->setSSvalue((double)level[listIndex]);
	myMeterWindow->setSNvalue((double)noise[listIndex]);
	myMeterWindow->setSQvalue((double)quality[listIndex]);
  }
  else { // no range
	myMeterWindow->setSSvalue(0);
	myMeterWindow->setSNvalue(0);
	myMeterWindow->setSQvalue(0);
  }

  if( managed == true ) {
	myMeterWindow->setNoRangeLevel( signalTuple.signal );
	myMeterWindow->setNoRangeNoise( signalTuple.noise );
	myMeterWindow->setNoRangeQuality(signalTuple.quality);
  }
  else { // ad hoc
	myMeterWindow->setNoRangeLevel( sigTupleAr[listIndex].signal );
	myMeterWindow->setNoRangeNoise( sigTupleAr[listIndex].noise );
	myMeterWindow->setNoRangeQuality( sigTupleAr[listIndex].quality );
  }
  currentMode = mode;
  myMeterWindow->redraw();
  Fl::repeat_timeout(pollInterval, updateDisplay);
}

//---------------------------------------------------------------------

int startInitialize( char *ifname ) 
{
  if( sigStat->Initialize(ifname, apscan ) == FAILURE ){
	return FAILURE;
  }
  else
	return SUCCESS;
}

//---------------------------------------------------------------------------  
void startWindow( )
{
  for( int i = 0; i < MAX_DISPLAY; i++ ) {
    ipList[i] = (char *)malloc( sizeof(char)*100 );
    if( ipList[i] == NULL ) {
      perror("Memory allocation failed\n");
      exit(FAILURE);
    }
  }

  getStat( sigStat );

  window = new openWindow( 240, 300, numberOfDisplay, "wifi scanner" );
  window->color(FL_BLACK);
  window->end();
 
  myMeterWindow = new meterWindow( 240, 300, "Signal Strength" );
  myMeterWindow->end();

  window->show();
  
  Fl::add_timeout(pollInterval, updateDisplay);
	for(;;) { 
	  Fl::wait();
	  if( change )
		break;
	}
	change = false;
  Fl::run();
}


//-----------------------------------------------------------------
Fl_Color genColor(int level)
{
  if ( level >=0 && level <= 25 )
	return FL_RED;
  else if( level > 25 && level <= 50 )
	return FL_BLUE;
  else if( level > 50 && level <= 75 )
	return FL_YELLOW;
  else
	return FL_GREEN;
}

//------------------------------------------------------------------

void list_cb( Fl_Widget *w, void *data )
{
  for( int i = 0; i < numberOfDisplay; i++ ) {
	if( strcmp( w->label() , ipList[i] ) == 0 ) {
	  listIndex = i;
	  if( adhoc == true ) {
	    myMeterWindow->setMac(macString[i]);
	    myMeterWindow->setIp(ipString[i]);
	    if( !noRange[i] ) {
		myMeterWindow->setSSvalue(level[i]);
		myMeterWindow->setSNvalue(noise[i]);
		myMeterWindow->setSQvalue(quality[i]);
	    }
	    else {
		myMeterWindow->setSSvalue( 0 );
		myMeterWindow->setSNvalue( 0 );
		myMeterWindow->setSQvalue( 0 );
	    }
	    myMeterWindow->setNoRangeLevel( sigTupleAr[listIndex].signal );
	    myMeterWindow->setNoRangeNoise( sigTupleAr[listIndex].noise );
	    myMeterWindow->setNoRangeQuality( sigTupleAr[listIndex].quality );
	  }
	  break;
	}
  }
  delete w->parent();
}

void closePopUp_cb( Fl_Widget *w, void *data )
{
  delete w;
}

void popList()
{
  // find list
  int listLength;
  if( managed == true )  
	listLength = 1;
  else
	listLength = numberOfDisplay ;

  int step = 20;
  int startY = 5;
  int length = listLength*step + 10;

  Fl_Window *popUp = new Fl_Window( 0,0,240,length,"Select from list:");
  popUp->color(FL_BLACK);
  popUp->callback(closePopUp_cb);
  popUp->set_modal();
  Fl_Button *menuItems[listLength];
  for( int i = 0; i < listLength; i++ ) {
	menuItems[i] = new Fl_Button(0, startY+i*step+5, 240, step, ipList[i]);
	menuItems[i]->color(255);
#ifdef IPAQ
	menuItems[i]->labelsize(6);
#else
	menuItems[i]->labelsize(8);
#endif
	menuItems[i]->callback(list_cb);
  }
  popUp->end();
  popUp->show();

}

//------------------------------------------------------------------

void msgok_cb(Fl_Widget *w, void *data)
{
	delete w->parent();
}

// window for giving message passed as the function parameter 
void message(char *msg)
{	  
  Fl_Window*msgWindow = new Fl_Window(240, 100, "Notice");
  msgWindow->set_modal();
  msgWindow->color(255);
  Fl_Box *msgBox = new Fl_Box( 5,5,150,50, msg);
  msgBox->color(255);
  msgBox->labelcolor(FL_BLUE);
  msgBox->labelfont(FL_HELVETICA_BOLD);
  Fl_Button *okButton = new Fl_Button( 140, 70, 50, 20, "OK");
  okButton->color(FL_BLUE);
  okButton->labelcolor(255);
  okButton->labelfont(FL_BOLD);
  okButton->callback(msgok_cb);
  msgWindow->end();
  msgWindow->show();
  Fl::run();
}

//----------------------------------------------------------------------
void yes_cb(Fl_Widget *w, void *data)
{
  sigStat->wifi_close();  /* let's close the interface, what say */
  exit(0);
}

void no_cb(Fl_Widget *w, void *data)
{
  delete w->parent();
}

// window for confirming quitting program 
void ask(char *msg)
{	  
  Fl_Window *askWindow = new Fl_Window(240, 100, "Confirm quitting");
  askWindow->set_modal();
  askWindow->color(255);
  Fl_Box *askBox = new Fl_Box( 5,5,150,50, msg);
  askBox->color(255);
  askBox->labelcolor(FL_BLUE);
  askBox->labelfont(FL_HELVETICA_BOLD);
  Fl_Button *yesButton = new Fl_Button( 80, 70, 50, 20, "YES");
  yesButton->color(FL_BLUE);
  yesButton->labelcolor(255);
  yesButton->labelfont(FL_BOLD);
  yesButton->callback(yes_cb);
  Fl_Button *noButton = new Fl_Button( 140, 70, 50, 20, "NO");
  noButton->color(FL_BLUE);
  noButton->labelcolor(255);
  noButton->labelfont(FL_BOLD);
  noButton->callback(no_cb);
  askWindow->end();
  askWindow->show();
}

//--------------------------------------------------------------

// Get the names of interfaces detected.
// pop up start window to prompt user to select
// interface name
int start(char *ifname)
{
  int result;
  //#define MAXIFS  40
  //char *iflist[MAXIFS];
  int count=0;

  for (int i = 0; i < MAX_DISPLAY; i++ ) {
    strcpy(macString[i], "");
    strcpy(ipString[i], "");
  }
  for ( int i = 0; i < MAXIFS; i++ ) {
    iflist[i] = (char *)malloc( sizeof(char) * 10 );
    strcpy( iflist[i], "" );
  }

  /* take user supplied ifname
  */
   if (strcmp( ifname, "" ) != 0 ) {
   	  result = startInitialize( ifname );
	  if( result == SUCCESS ) {
	    strcpy( opInfo.ifname, ifname );
		startWindow();
		return(SUCCESS);
	  }
  }

  /* else try and guess the interface name
  */

  // get names of devices
  sigStat->wifi_getifs( &count, iflist, MAXIFS);
  
  if( count == -1 || count == 0) {
    message("Unable to get interface");
    exit(FAILURE);
  }

  // more than one interface, find one which works
  for( int i = 0; i < count; i++ ) {
	strcpy( ifname, iflist[i] );
	int result = startInitialize( ifname );
	if( result == SUCCESS ) {
		startWindow();
		return(SUCCESS);
	}
  }
  message("No interface works");
   exit( FAILURE );
}

void
usage()
{
	printf("wscan [-P] [-A] -i <ifname>\n");
	printf("-P - prism2 interprolation of signal strength\n");
	printf("-A - scan AP mode\n");
	exit(1);
}

/**************************MAIN**************************************/

int main( int argc, char **argv )
{
  int ch;

  strcpy(ifname, "");

  optarg = NULL;
  while (((ch = getopt(argc, argv, "i:PA")) != -1)) {
    switch(ch) {
    case 'A':
      apscan = 1;
      break;
    case 'i':
      strcpy(ifname, optarg);
      break;
    case 'P':		/* prism2 flag */
      prism2 = 1;
      break;
    default:
      usage();
      break;
    }
  } 
#ifndef FREEBSD
  if (apscan) {
    printf("sorry - apscan mode not yet implemented in linux\n");
    exit(0);
  }
#endif
  argc -= optind;
  argv += optind;
    
  listIndex = 0;
  sigStat = new cSigStat();
  pollInterval = 2.0;
  myCounterWindow = new counterWindow( 190, 80, "Setting polling Interval");
  myCounterWindow->end();
  start(ifname);
}
